'use strict';

var t = require('typanion');
var advanced_Command = require('../advanced/Command.js');
var advanced_Cli = require('../advanced/Cli.js');
require('../constants.js');
var advanced_builtins_definitions = require('../advanced/builtins/definitions.js');
var advanced_builtins_help = require('../advanced/builtins/help.js');
var advanced_builtins_version = require('../advanced/builtins/version.js');
var advanced_options_Array = require('../advanced/options/Array.js');
var advanced_options_Boolean = require('../advanced/options/Boolean.js');
var advanced_options_Proxy = require('../advanced/options/Proxy.js');
var advanced_options_Rest = require('../advanced/options/Rest.js');
var advanced_options_String = require('../advanced/options/String.js');

function _interopNamespaceDefault(e) {
    var n = Object.create(null);
    if (e) {
        Object.keys(e).forEach(function (k) {
            if (k !== 'default') {
                var d = Object.getOwnPropertyDescriptor(e, k);
                Object.defineProperty(n, k, d.get ? d : {
                    enumerable: true,
                    get: function () { return e[k]; }
                });
            }
        });
    }
    n.default = e;
    return Object.freeze(n);
}

var t__namespace = /*#__PURE__*/_interopNamespaceDefault(t);

class YarnDefaultRun extends advanced_Command.Command {
    constructor() {
        super(...arguments);
        this.scriptName = advanced_options_String.String();
        this.rest = advanced_options_Proxy.Proxy();
    }
    async execute() {
        return await this.cli.run([`run`, this.scriptName, ...this.rest], {});
    }
}
const isPositiveInteger = t__namespace.applyCascade(t__namespace.isNumber(), [
    t__namespace.isInteger(),
    t__namespace.isAtLeast(1),
]);
class YarnInstall extends advanced_Command.Command {
    constructor() {
        super(...arguments);
        this.frozenLockfile = advanced_options_Boolean.Boolean(`--frozen-lockfile`, false);
        this.maxRetries = advanced_options_String.String(`--max-retries`, `0`, { validator: isPositiveInteger });
    }
    async execute() {
        this.context.stdout.write(`Running an install: ${this.context.cwd}, with ${this.maxRetries} max retries\n`);
    }
}
YarnInstall.paths = [advanced_Command.Command.Default, [`install`]];
class YarnRunListing extends advanced_Command.Command {
    constructor() {
        super(...arguments);
        this.json = advanced_options_Boolean.Boolean(`--json`, false);
    }
    async execute() {
        this.context.stdout.write(`Listing all the commands (json = ${this.json})\n`);
    }
}
YarnRunListing.paths = [[`run`]];
class YarnRunExec extends advanced_Command.Command {
    constructor() {
        super(...arguments);
        this.scriptName = advanced_options_String.String();
        this.rest = advanced_options_Proxy.Proxy();
    }
    async execute() {
        this.context.stdout.write(`Executing a script named ${this.scriptName} ${this.rest}\n`);
    }
}
YarnRunExec.usage = advanced_Command.Command.Usage({
    category: `Script-related commands`,
});
YarnRunExec.paths = [[`run`]];
// eslint-disable-next-line arca/no-default-export
class YarnAdd extends advanced_Command.Command {
    constructor() {
        super(...arguments);
        this.dev = advanced_options_Boolean.Boolean(`-D,--dev`, false, { description: `Use dev mode` });
        this.peer = advanced_options_Boolean.Boolean(`-P,--peer`, false, { description: `Use peer mode` });
        this.exact = advanced_options_Boolean.Boolean(`-E,--exact`, false, { description: `Don't add ^ nor ~` });
        this.tilde = advanced_options_Boolean.Boolean(`-T,--tilde`, false, { description: `Use ~` });
        this.caret = advanced_options_Boolean.Boolean(`-C,--caret`, false, { description: `Use ^` });
        this.pkgs = advanced_options_Rest.Rest({ required: 1 });
    }
    async execute() {
        if (this.dev) {
            this.context.stdout.write(`Adding a dev dependency\n`);
        }
        else if (this.peer) {
            this.context.stdout.write(`Adding a peer dependency\n`);
        }
        else {
            this.context.stdout.write(`Adding a dependency\n`);
        }
    }
}
YarnAdd.schema = [
    t__namespace.hasMutuallyExclusiveKeys([`dev`, `peer`]),
    t__namespace.hasMutuallyExclusiveKeys([`exact`, `tilde`, `caret`]),
];
YarnAdd.usage = advanced_Command.Command.Usage({
    description: `
      add dependencies to the project
    `,
    details: `
      This command adds a package to the package.json for the nearest workspace.

      - The package will by default be added to the regular \`dependencies\` field, but this behavior can be overriden thanks to the \`-D,--dev\` flag (which will cause the dependency to be added to the \`devDependencies\` field instead) and the \`-P,--peer\` flag (which will do the same but for \`peerDependencies\`).
      - If the added package **doesn't** specify a range at all its \`latest\` tag will be resolved and the returned version will be used to generate a new semver range (using the \`^\` modifier by default, or the \`~\` modifier if \`-T,--tilde\` is specified, or no modifier at all if \`-E,--exact\` is specified). Two exceptions to this rule: the first one is that if the package is a workspace then its local version will be used, and the second one is that if you use \`-P,--peer\` the default range will be \`*\` and won't be resolved at all.
      - If the added package specifies a tag range (such as \`latest\` or \`rc\`), Yarn will resolve this tag to a semver version and use that in the resulting package.json entry (meaning that \`yarn add foo@latest\` will have exactly the same effect as \`yarn add foo\`).

      If the \`-i,--interactive\` option is used (or if the \`preferInteractive\` settings is toggled on) the command will first try to check whether other workspaces in the project use the specified package and, if so, will offer to reuse them.

      If the \`--cached\` option is used, Yarn will preferably reuse the highest version already used somewhere within the project, even if through a transitive dependency.

      For a compilation of all the supported protocols, please consult the dedicated page from our website: http://example.org.
    `,
    examples: [[
            `Add the latest version of a package`,
            `$0 add lodash`,
        ], [
            `Add a specific version of a package`,
            `$0 add lodash@3.0.0`,
        ]],
});
YarnAdd.paths = [[`add`]];
class YarnRemove extends advanced_Command.Command {
    constructor() {
        super(...arguments);
        this.packages = advanced_options_Rest.Rest();
    }
    async execute() {
    }
}
YarnRemove.usage = advanced_Command.Command.Usage({
    description: `remove dependencies from the project`,
    details: `
      This command will remove the specified packages from the current workspace. If the \`-A,--all\` option is set, the operation will be applied to all workspaces from the current project.
    `,
    examples: [[
            `Remove a dependency from the current project`,
            `$0 remove lodash`,
        ], [
            `Remove a dependency from all workspaces at once`,
            `$0 remove lodash --all`,
        ]],
});
YarnRemove.paths = [[`remove`]];
class YarnWorkspacesForeachCommand extends advanced_Command.Command {
    constructor() {
        super(...arguments);
        this.include = advanced_options_Array.Array(`--include`);
    }
    async execute() {
    }
}
YarnWorkspacesForeachCommand.paths = [[`workspaces`, `foreach`]];
const cli = new advanced_Cli.Cli({
    binaryLabel: `Yarn Project Manager`,
    binaryName: `yarn`,
    binaryVersion: `0.0.0`,
});
cli.register(advanced_builtins_definitions.DefinitionsCommand);
cli.register(advanced_builtins_help.HelpCommand);
cli.register(advanced_builtins_version.VersionCommand);
cli.register(YarnDefaultRun);
cli.register(YarnInstall);
cli.register(YarnRemove);
cli.register(YarnRunListing);
cli.register(YarnRunExec);
cli.register(YarnAdd);
cli.register(YarnWorkspacesForeachCommand);
cli.runExit(process.argv.slice(2), {
    cwd: process.cwd(),
});

module.exports = YarnAdd;
